/*******************(C) COPYRIGHT 2021 Masses-Chip ******************************
*Project Name        : M9F680_I2C_MASTER.prj
* File Name          : M9F680_I2C_MASTER_WR.C
* Author             : MASSES CHIP
* Version            : V2.00
* Date               : 2021/10/08
* Web    			 : www.masses-chip.com
* FAE                : Luo
;					 : QQ：411680975
********************************************************************************
;Option:16M/16T,WDT-Enable-288ms,RST-IO,BOR 2.4V 
;说明：本范例是作为I2C主机(SCL:IOB0 SDA:IOB1)
;主机写：主机向从机发送17个数据，第一个数据为要写入RAM区的地址，其他数据为写入的数据
;主机读：主机先发送要读出RAM区的地址，再读16个数据依次放在RAM(0x0010-0x001F)中
******************************************************************************/

#include "zc.h"
#include "MASSESCHIP_DEFINE.H"

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
#define      STA_M_START    	0X08    //主机发送开始信号完成
#define      STA_M_RSTART    	0X10    //主机发送重新开始信号完成
#define      STA_M_W_ADRACK  	0X18   	//主机发送地址应答
#define      STA_M_W_ADRNACK 	0X20   	//主机发送地址不应答
#define      STA_M_W_DATACK   	0X28    //主机发送数据应答
#define      STA_M_W_DATNACK  	0X30    //主机发送数据不应答
#define		 STA_M_W_ADRACK_LOST 0x38	//主机仲裁失败
#define      STA_M_R_ADRACK   	0X40    //主机接收地址应答
#define      STA_M_R_ADRNACK  	0X48    //主机接收地址不应答
#define      STA_M_R_DATACK   	0X50    //主机接收数据应答
#define      STA_M_R_DATNACK  	0X58    //主机接收数据不应答
#define		 STA_M_W_ERR		0x00	//总线错误
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 


uchar Number;
uchar I2C_DATA;

void I2C_Start(void);						//起始指令
void Send_Add(uchar DATA);					//发送地址
void Send_Data(uchar DATA);					//发送数据
void I2C_STOP(void);						//结束指令
void I2C_Rstart(void);						//重复起始指令
void Receive_Data(void);					//接收数据
void STOP_Receive(void);					//停止接收数据
void STATE_Err(void);						//状态错误
void STATE_W_ADRNACK(void);					//写地址未应答
void STATE_WDATNACK(void);					//写数据未应答
/*===============================================================================
**********************************I2C初始化***********************************
1、端口设置：设置IOB0、IOB1为上拉、数字口，设置端口复用功能IOB0---SCL,IOB1---SDA
2、I2C设置：作主机
3、开启总中断
===============================================================================*/	
void I2C_INIT(void)
{
//1、端口设置
	PUB0 = 1;
	PUB1 = 1;
	ANSB = 0;
	MPSCL = 0x10;
	MPSDA = 0x11;
	
//2、I2C设置
	I2CCR = 0x40;
	
//3、开中断
	GIEH = 1;
}

//===============================================================================
//*********************************I2C主机写*************************************
//主机写：主机向从机发送17个数据，第一个数据为要写入RAM区的地址，其他数据为写入的数据
//===============================================================================			
void I2C_Master_Write(void)
{
	I2C_Start();						//起始指令
	Send_Add(0xA0);
	Send_Data(0x10);
	for(I2C_DATA = 0;I2C_DATA < 0x10;)
	{
		Send_Data(I2C_DATA++);
	}
	I2C_STOP();							//结束指令
}


/*===============================================================================
*********************************I2C主机读*************************************
主机读：主机先发送要读出RAM区的地址，再读16个数据依次放在RAM(0x010)中
================================================================================*/
void I2C_Master_Read(void)
{
	FSR1H = 0;
	FSR1L = 0x10;						//存放地址
	Number = 0x10;						//读16位数据
	I2C_Start();						//起始指令
	Send_Add(0xA0);						//发送从机地址写
	Send_Data(0x20);					//发送要读出的RAM区地址
	I2C_Rstart();						//重复起始指令
	Send_Add(0xA1);						//发送从机地址读
	for(;--Number != 0;)
	{
		Receive_Data();
	}
	STOP_Receive();
	I2C_STOP();
}

/******************************************************************************
;=================================起始信号=====================================
******************************************************************************/	
void I2C_Start()
{
	I2CST = 1;
	while(!INTF);
	if(I2CSTA == STA_M_START)
	{
		I2CST = 0;
	}
	else
	{
		STATE_Err();
	}
}

/********************************************************************************
;=================================重复起始信号===================================
********************************************************************************/	
void I2C_Rstart(void)
{
	I2CST = 1;
	I2C0IE = 1;
	while(INTF);
	while(!INTF);
	if(I2CSTA == STA_M_RSTART)
	{
		I2CST = 0;
	}
	else
	{
		STATE_Err();
	}
}

/******************************************************************************
;============================发送从机地址读、写================================
******************************************************************************/			
void Send_Add(uchar DATA)
{
	I2CDATA = DATA;
	I2C0IE = 1;								//开中断
	while(INTF);
	while(!INTF);							//等待数据发送完成清除中断
	NOP();
	if((DATA&0x01) != 1)						//写
	{
		if(I2CSTA == STA_M_W_ADRNACK)		//判断发送地址是否应答
		{
			STATE_W_ADRNACK();
		}
		else
		{
			if(I2CSTA != STA_M_W_ADRACK)
			{
				STATE_Err();
			}
		}
	}
	else									//读
	{
		ACK = 1;
		if(I2CSTA == STA_M_R_ADRNACK)
		{
			STATE_W_ADRNACK();				//未应答
		}
		else
		{
			if(I2CSTA != STA_M_R_ADRACK)
			{
				STATE_Err();
			}
		}
	}
}

/******************************************************************************
;===============================发送数据=======================================
******************************************************************************/			
void Send_Data(uchar DATA)
{
	I2CDATA = DATA;
	I2C0IE = 1;								//开中断
	while(INTF);
	while(!INTF);							//等待数据发送完成清除中断
	if(I2CSTA == STA_M_W_DATNACK)
	{
		STATE_WDATNACK();
	}
	else
	{
		if(I2CSTA != STA_M_W_DATACK)
		{
			STATE_Err();
		}
	}
}

/******************************************************************************
;===============================接收数据=======================================
******************************************************************************/			
void Receive_Data(void)
{
	I2C0IE = 1;								//开中断
	while(INTF);
	while(!INTF);							//等待数据发送完成清除中断
	if(I2CSTA == STA_M_R_DATACK)
	{
		POSTINC1 = I2CDATA;
	}
	else
	{
		STATE_Err();
	}
}

/******************************************************************************
;=================================结束信号=====================================
******************************************************************************/
void I2C_STOP(void)
{
	I2CSTO = 1;
	I2C0IE = 1;
	while(INTF);
}

/******************************************************************************
;=================================停止接收=====================================
******************************************************************************/	
void STOP_Receive(void)
{
	ACK = 0;								//关闭应答
	I2C0IE = 1;
	while(INTF);
	while(!INTF);
	if(I2CSTA == STA_M_R_DATNACK)
	{
		INDF1 = I2CDATA;
	}
	else
	{
		STATE_Err();
	}
}

/******************************************************************************
;=================================状态错误=====================================
******************************************************************************/	
void STATE_Err(void)
{
	while(1);
}

/******************************************************************************
;=================================写地址未应答=================================
******************************************************************************/	
void STATE_W_ADRNACK(void)
{
	while(1);
}

/******************************************************************************
;=================================写数据未应答=================================
******************************************************************************/	
void STATE_WDATNACK(void)
{
	while(1);
}
